@@ -1,10 +1,7 @@
 use crate::common;
 use proc_macro2::TokenStream;
-use quote::{quote, quote_spanned};
-use syn::{
-    spanned::Spanned, Data, DataEnum, DataStruct, DeriveInput, Error, Fields, FieldsNamed, Result,
-    Visibility,
-};
+use quote::quote;
+use syn::{Data, DataEnum, DataStruct, DeriveInput, Error, Fields, FieldsNamed, Result};
 
 pub fn derive(node: &DeriveInput) -> Result<TokenStream> {
     match &node.data {
@@ -66,7 +63,6 @@ fn derive_c_struct(
 ) -> Result<TokenStream> {
     let ty = &node.ident;
     let (impl_generics, ty_generics, where_clause) = node.generics.split_for_impl();
-    let deserialize_trait = spanned_deserialize_trait(node);
 
     let fn_body = {
         let mut members = vec![];
@@ -119,10 +115,10 @@ fn derive_c_struct(
         }
 
         quote! {
-            let __len = __deserializer
-                .deserialize_token()?
-                .to_map()
-                .ok_or(::msgpack_schema::ValidationError)?;
+            let __len = match __deserializer.deserialize_token()? {
+                ::msgpack_schema::Token::Map(len) => len,
+                _ => return Err(::msgpack_schema::ValidationError.into()),
+            };
 
             #( #init )*
             for _ in 0..__len {
@@ -142,7 +138,7 @@ fn derive_c_struct(
 
     let gen = quote! {
         #[allow(unused_qualifications)]
-        impl #impl_generics #deserialize_trait for #ty #ty_generics #where_clause {
+        impl #impl_generics ::msgpack_schema::Deserialize for #ty #ty_generics #where_clause {
             fn deserialize(__deserializer: &mut ::msgpack_schema::Deserializer) -> ::std::result::Result<Self, ::msgpack_schema::DeserializeError> {
                 #fn_body
             }
@@ -159,7 +155,6 @@ fn derive_newtype_struct(
 ) -> Result<TokenStream> {
     let ty = &node.ident;
     let (impl_generics, ty_generics, where_clause) = node.generics.split_for_impl();
-    let deserialize_trait = spanned_deserialize_trait(node);
 
     let fn_body = quote! {
         __deserializer.deserialize().map(Self)
@@ -167,7 +162,7 @@ fn derive_newtype_struct(
 
     let gen = quote! {
         #[allow(unused_qualifications)]
-        impl #impl_generics #deserialize_trait for #ty #ty_generics #where_clause {
+        impl #impl_generics ::msgpack_schema::Deserialize for #ty #ty_generics #where_clause {
             fn deserialize(__deserializer: &mut ::msgpack_schema::Deserializer) -> ::std::result::Result<Self, ::msgpack_schema::DeserializeError> {
                 #fn_body
             }
@@ -180,7 +175,6 @@ fn derive_newtype_struct(
 fn derive_enum(node: &DeriveInput, enu: &DataEnum) -> Result<TokenStream> {
     let ty = &node.ident;
     let (impl_generics, ty_generics, where_clause) = node.generics.split_for_impl();
-    let deserialize_trait = spanned_deserialize_trait(node);
 
     let fn_body = {
         let mut clauses = vec![];
@@ -259,7 +253,7 @@ fn derive_enum(node: &DeriveInput, enu: &DataEnum) -> Result<TokenStream> {
 
     let gen = quote! {
         #[allow(unused_qualifications)]
-        impl #impl_generics #deserialize_trait for #ty #ty_generics #where_clause {
+        impl #impl_generics ::msgpack_schema::Deserialize for #ty #ty_generics #where_clause {
             fn deserialize(__deserializer: &mut ::msgpack_schema::Deserializer) -> ::std::result::Result<Self, ::msgpack_schema::DeserializeError> {
                 #fn_body
             }
@@ -272,7 +266,6 @@ fn derive_enum(node: &DeriveInput, enu: &DataEnum) -> Result<TokenStream> {
 fn derive_untagged_enum(node: &DeriveInput, enu: &DataEnum) -> Result<TokenStream> {
     let ty = &node.ident;
     let (impl_generics, ty_generics, where_clause) = node.generics.split_for_impl();
-    let deserialize_trait = spanned_deserialize_trait(node);
 
     let fn_body = {
         let mut members = vec![];
@@ -329,7 +322,7 @@ fn derive_untagged_enum(node: &DeriveInput, enu: &DataEnum) -> Result<TokenStrea
 
     let gen = quote! {
         #[allow(unused_qualifications)]
-        impl #impl_generics #deserialize_trait for #ty #ty_generics #where_clause {
+        impl #impl_generics ::msgpack_schema::Deserialize for #ty #ty_generics #where_clause {
             fn deserialize(__deserializer: &mut ::msgpack_schema::Deserializer) -> ::std::result::Result<Self, ::msgpack_schema::DeserializeError> {
                 #fn_body
             }
@@ -346,7 +339,6 @@ fn derive_untagged_c_struct(
 ) -> Result<TokenStream> {
     let ty = &node.ident;
     let (impl_generics, ty_generics, where_clause) = node.generics.split_for_impl();
-    let deserialize_trait = spanned_deserialize_trait(node);
 
     let fn_body = {
         let mut members = vec![];
@@ -375,10 +367,10 @@ fn derive_untagged_c_struct(
         }
 
         quote! {
-            let __len = __deserializer
-                .deserialize_token()?
-                .to_array()
-                .ok_or(::msgpack_schema::ValidationError)?;
+            let __len = match __deserializer.deserialize_token()? {
+                Token::Array(len) => len,
+                _ => return Err(::msgpack_schema::ValidationError.into()),
+            };
 
             if __len != #len {
                 return Err(::msgpack_schema::ValidationError.into());
@@ -392,7 +384,7 @@ fn derive_untagged_c_struct(
 
     let gen = quote! {
         #[allow(unused_qualifications)]
-        impl #impl_generics #deserialize_trait for #ty #ty_generics #where_clause {
+        impl #impl_generics ::msgpack_schema::Deserialize for #ty #ty_generics #where_clause {
             fn deserialize(__deserializer: &mut ::msgpack_schema::Deserializer) -> ::std::result::Result<Self, ::msgpack_schema::DeserializeError> {
                 #fn_body
             }
@@ -401,24 +393,3 @@ fn derive_untagged_c_struct(
 
     Ok(gen)
 }
-
-fn spanned_deserialize_trait(input: &DeriveInput) -> TokenStream {
-    let path = {
-        let span = match &input.vis {
-            Visibility::Public(vis) => vis.pub_token.span(),
-            Visibility::Crate(vis) => vis.crate_token.span(),
-            Visibility::Restricted(vis) => vis.pub_token.span(),
-            Visibility::Inherited => match &input.data {
-                Data::Struct(data) => data.struct_token.span(),
-                Data::Enum(data) => data.enum_token.span(),
-                Data::Union(data) => data.union_token.span(),
-            },
-        };
-        quote_spanned!(span => ::msgpack_schema::)
-    };
-    let serialize = {
-        let span = input.ident.span();
-        quote_spanned!(span => Deserialize)
-    };
-    quote!(#path #serialize)
-}
